import cv2
import imutils as imutils
import serial


def to_position(coords):
    # extract the x and y values
    x, y = coords

    if x >= x_dif_offset or x <= -x_dif_offset:
        if y_dif_offset >= y >= -y_dif_offset:
            if x > 0:
                return "EAST"
            else:
                return "WEST"
        elif y <= -y_dif_offset:
            if x > 0:
                return "NORTH_EAST"
            else:
                return "NORTH_WEST"
        elif y >= y_dif_offset:
            if x > 0:
                return "SOUTH_EAST"
            else:
                return "SOUTH_WEST"
    else:
        if y >= y_dif_offset:
            return "SOUTH"
        elif y <= -y_dif_offset:
            return "NORTH"

    return "CENTER"


y_dif_offset = 50
x_dif_offset = 50

frame_width = 800
frame_height = 400

ref_width = 50
ref_height = 50

# bottom left corner for centered (reference) rectangle
ref_btm = (int(frame_width / 2 - ref_width / 2), int(frame_height / 2 - ref_height / 2))

# top right corner for centered (reference) rectangle
ref_top = (int(frame_width / 2 + ref_width / 2), int(frame_height / 2 + ref_height / 2))

# coordinates of the reference rectangle
ref_center = (frame_width / 2, frame_height / 2)

# initialize classifier
classifier = cv2.CascadeClassifier(cv2.data.haarcascades
                                   + 'haarcascade_frontalface_default.xml')
# initialize camera
camera = cv2.VideoCapture(0)

# initialize serial com
to_board_serial = serial.Serial('/dev/cu.usbmodem14101', 9600)

while True:
    # extract current frame
    _, frame = camera.read()

    # resize frame to fit given dimensions
    frame = imutils.resize(frame, width=800, height=400)

    # mirror the image for better orientation
    frame = cv2.flip(frame, 1)

    # convert frame to its grayscale version
    gray_frame = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)

    # detect face
    faces = classifier.detectMultiScale(
        gray_frame,
        scaleFactor=1.1,
        minNeighbors=4,
        minSize=(20, 20),
        flags=cv2.CASCADE_SCALE_IMAGE
    )

    # draw reference rectangle
    cv2.rectangle(frame, ref_btm, ref_top, (0, 0, 255), 2)

    # check if any faces were detected
    if len(faces) > 0:
        # draw rectangle around the first face
        (x, y, w, h) = faces[0]
        cv2.rectangle(frame, (x, y), (x + w, y + h), (0, 255, 0), 2)

        # compute center of face
        center = (x + w / 2, y + h / 2)

        # compute difference between face's center and ref's center
        dif = (center[0] - ref_center[0], center[1] - ref_center[1])

        # convert coord. dif to an eye position
        serial_str = to_position(dif) + '\n'

        # send eye position to board
        to_board_serial.write(serial_str.encode('utf-8'))

    # Display frame
    cv2.imshow('Video', frame)

    # wait before proceeding to next frame
    cv2.waitKey(1)
